//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/FTDecayFunctionItems.h
//! @brief     Defines FTDecayFunction1DItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_FTDECAYFUNCTIONITEMS_H
#define BORNAGAIN_GUI_MODEL_ITEM_FTDECAYFUNCTIONITEMS_H

#include "GUI/Model/Types/DoubleDescriptor.h"
#include "GUI/Model/Types/DoubleProperty.h"
#include "Sample/Correlations/FTDecay1D.h"
#include "Sample/Correlations/FTDecay2D.h"
#include <memory>

class Serializer;

class FTDecayFunction1DItem {
public:
    virtual ~FTDecayFunction1DItem() = default;
    virtual std::unique_ptr<IFTDecayFunction1D> createFTDecayFunction() const = 0;
    virtual void serialize(Serializer& s);

    DoubleDescriptor decayLength() const;
    virtual DoubleDescriptors valueDescriptors() const;

protected:
    FTDecayFunction1DItem();
    DoubleProperty m_decayLength;
};

class FTDecayFunction1DCauchyItem : public FTDecayFunction1DItem {
public:
    std::unique_ptr<IFTDecayFunction1D> createFTDecayFunction() const override;
};

class FTDecayFunction1DGaussItem : public FTDecayFunction1DItem {
public:
    std::unique_ptr<IFTDecayFunction1D> createFTDecayFunction() const override;
};

class FTDecayFunction1DTriangleItem : public FTDecayFunction1DItem {
public:
    std::unique_ptr<IFTDecayFunction1D> createFTDecayFunction() const override;
};

class FTDecayFunction1DVoigtItem : public FTDecayFunction1DItem {
public:
    FTDecayFunction1DVoigtItem();
    std::unique_ptr<IFTDecayFunction1D> createFTDecayFunction() const override;
    void serialize(Serializer& s) override;

    DoubleDescriptor eta() const;
    DoubleDescriptors valueDescriptors() const override;

private:
    DoubleProperty m_eta;
};

// --------------------------------------------------------------------------------------------- //

class FTDecayFunction2DItem {
public:
    virtual ~FTDecayFunction2DItem() = default;
    virtual std::unique_ptr<IFTDecayFunction2D> createFTDecayFunction() const = 0;
    virtual void serialize(Serializer& s);

    DoubleDescriptor decayLengthX() const;
    void setDecayLengthX(double decay_length_x);

    DoubleDescriptor decayLengthY() const;
    void setDecayLengthY(double decay_length_y);

    DoubleDescriptor gamma() const;
    void setGamma(double gamma);

    virtual DoubleDescriptors valueDescriptors() const;

protected:
    FTDecayFunction2DItem();

    DoubleProperty m_decayLengthX;
    DoubleProperty m_decayLengthY;
    DoubleProperty m_gamma;
};

class FTDecayFunction2DCauchyItem : public FTDecayFunction2DItem {
public:
    std::unique_ptr<IFTDecayFunction2D> createFTDecayFunction() const override;
};

class FTDecayFunction2DGaussItem : public FTDecayFunction2DItem {
public:
    std::unique_ptr<IFTDecayFunction2D> createFTDecayFunction() const override;
};

class FTDecayFunction2DVoigtItem : public FTDecayFunction2DItem {
public:
    FTDecayFunction2DVoigtItem();
    std::unique_ptr<IFTDecayFunction2D> createFTDecayFunction() const override;
    void serialize(Serializer& s) override;

    void setEta(double eta);
    DoubleDescriptor eta() const;

    virtual DoubleDescriptors valueDescriptors() const override;

private:
    DoubleProperty m_eta;
};

#endif // BORNAGAIN_GUI_MODEL_ITEM_FTDECAYFUNCTIONITEMS_H
