//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/MesoCrystalItem.h
//! @brief     Defines class MesoCrystalItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_MESOCRYSTALITEM_H
#define BORNAGAIN_GUI_MODEL_ITEM_MESOCRYSTALITEM_H

#include "GUI/Model/Group/SelectionProperty.h"
#include "GUI/Model/Item/FormFactorItems.h"
#include "GUI/Model/Item/ItemWithParticles.h"
#include "GUI/Model/Types/VectorProperty.h"
#include "Sample/Lattice/Lattice3D.h"
#include <memory>

class IBornFF;
class IParticle;
class MesoCrystal;

class MesoCrystalItem : public ItemWithParticles {
public:
    MesoCrystalItem();
    void serialize(Serializer& s) override;

    std::unique_ptr<MesoCrystal> createMesoCrystal() const;

    Lattice3D getLattice() const;
    std::unique_ptr<IParticle> getBasis() const;

    std::unique_ptr<IBornFF> getOuterShape() const;
    template <typename T> T* setOuterShapeType();

    SelectionDescriptor<FormFactorItem*> outerShape() const;
    void setOuterShape(FormFactorItem* p);

    ItemWithParticles* basisParticle() const;
    void setBasis(ItemWithParticles* basis);

    void setVectorA(const R3& vector_a);
    void setVectorB(const R3& vector_b);
    void setVectorC(const R3& vector_c);

    VectorDescriptor vectorA() const;
    VectorDescriptor vectorB() const;
    VectorDescriptor vectorC() const;

    QVector<ItemWithParticles*> containedItemsWithParticles() const override;

private:
    VectorProperty m_vectorA;
    VectorProperty m_vectorB;
    VectorProperty m_vectorC;
    SelectionProperty<FormFactorItem*> m_outerShape;
    SelectionProperty<ItemWithParticles*> m_basisParticle;
};

template <typename T> T* MesoCrystalItem::setOuterShapeType()
{
    auto* p = new T();
    setOuterShape(p);
    return p;
}

#endif // BORNAGAIN_GUI_MODEL_ITEM_MESOCRYSTALITEM_H
