//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Types/DoubleProperty.cpp
//! @brief     Implements class DoubleProperty
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Types/DoubleProperty.h"
#include <QUuid>

void DoubleProperty::init(const QString& label, const QString& tooltip, double value,
                          const variant<QString, Unit>& unit, const QString& persistentTag)
{
    init(label, tooltip, value, unit, 3, RealLimits::nonnegative(), persistentTag);
}

void DoubleProperty::init(const QString& label, const QString& tooltip, double value,
                          const variant<QString, Unit>& unit, int decimals,
                          const RealLimits& limits, const QString& persistentTag)
{
    m_value = value;
    m_decimals = decimals;
    m_persistentTag = persistentTag;
    m_uid = QUuid::createUuid().toString(QUuid::WithoutBraces);

    m_descriptor.label = label;
    m_descriptor.tooltip = tooltip;
    m_descriptor.decimals = m_decimals;
    m_descriptor.limits = limits;
    m_descriptor.unit = unit;
    m_descriptor.set = [=](double v) { m_value = v; };
    m_descriptor.get = [=] { return m_value; };
    m_descriptor.path = [=] { return m_uid; };
}
